#ifndef CUETRACKWIDGET_H
#define CUETRACKWIDGET_H

#include <QWidget>
#include <QLineEdit>
#include <QSpinBox>
#include <QComboBox>
#include <QFormLayout>
#include <QCheckBox>
#include <QLabel>
#include <QPushButton>
#include <QDialog>
#include "defines.h"
#include "clickabledoublespinbox.h"

class CueTrackWidget : public QWidget {
    Q_OBJECT

public:
    explicit CueTrackWidget(QWidget *parent = nullptr);
    CueTrack track;
    CueTrack saveCueTrack() const;
    void loadCueTrack(const CueTrack &cueTrack);
    int getVolume() const { return volumeSpin->value() * 2.55f * 255.0f; }
    int getPan() const { return (panSpin->value() * 128) + 128; }
    int getPitch() const { return pitchSpin->value() * 128; }
    int getBus1() const { return bus1Spin->value() * 2.55f * 255.0f; }
    int getBus2() const { return bus2Spin->value() * 2.55f * 255.0f; }
	Status getStatus() const { return static_cast<Status>(statusCombo->currentIndex()); }
	QString getFilePath() const { return filePathEdit->text(); }
    void setFilePath(QString text) const { filePathEdit->setText(text);};
	int getFadeIn() const { return fadeInSpin->value(); }
	int getFadeOut() const { return fadeOutSpin->value(); }
	int getWaitIn() const { return waitInSpin->value(); }
	int getWaitOut() const { return waitOutSpin->value(); }
	bool getStopAtEnd() const { return stopAtEndCheck->isChecked(); }
	QString getName() const { return nameEdit->text(); }
	QString getDescription() const { return descriptionEdit->text(); }
	int getUserNumber() const { return userNumberSpin->value(); }
	int getEntryPoint() const { return entryPointSpin->value(); }
	int getExitPoint() const { return exitPointSpin->value(); }
    void setStyle(bool highlight);

    QCheckBox* active;
    QSpinBox* audioLayer;
    QLineEdit* filePathEdit;
    ClickableDoubleSpinBox* volumeSpin;
    ClickableDoubleSpinBox* panSpin;
    ClickableDoubleSpinBox* pitchSpin;
    ClickableDoubleSpinBox* bus1Spin;
    ClickableDoubleSpinBox* bus2Spin;
    QComboBox* statusCombo;
    QSpinBox* fadeOutSpin;
    QSpinBox* fadeInSpin;
    QSpinBox* waitInSpin;
    QSpinBox* waitOutSpin;
    QCheckBox* stopAtEndCheck;
    QLineEdit* nameEdit;
    QLineEdit* descriptionEdit;
    QSpinBox* userNumberSpin;
    QSpinBox* entryPointSpin;
    QSpinBox* exitPointSpin;
	QPushButton* editButton;
    
private:
    void setupUi();
    void setupStatusCombo();

private slots:
    void editButtonClicked();
/*
	#include <QFile>
	#include <QTextStream>

	bool CueTrack::saveToFile(const QString& filePath) const {
		QFile file(filePath);
		if (!file.open(QIODevice::WriteOnly | QIODevice::Text)) {
			return false; // No se pudo abrir el archivo para escritura
		}

		QTextStream out(&file);

		// Escribir los datos en el archivo
		out << "Volume: " << getVolume() << "\n";
		out << "Pan: " << getPan() << "\n";
		out << "Pitch: " << getPitch() << "\n";
		out << "Bus1: " << getBus1() << "\n";
		out << "Bus2: " << getBus2() << "\n";
		out << "Status: " << static_cast<int>(getStatus()) << "\n";
		out << "FilePath: " << getFilePath() << "\n";
		out << "FadeIn: " << getFadeIn() << "\n";
		out << "FadeOut: " << getFadeOut() << "\n";
		out << "WaitIn: " << getWaitIn() << "\n";
		out << "WaitOut: " << getWaitOut() << "\n";
		out << "StopAtEnd: " << getStopAtEnd() << "\n";
		out << "Name: " << getName() << "\n";
		out << "Description: " << getDescription() << "\n";
		out << "UserNumber: " << getUserNumber() << "\n";
		out << "EntryPoint: " << getEntryPoint() << "\n";
		out << "ExitPoint: " << getExitPoint() << "\n";

		file.close();
		return true;
	}
	#include <QFile>
	#include <QTextStream>

	bool CueTrack::loadFromFile(const QString& filePath) {
		QFile file(filePath);
		if (!file.open(QIODevice::ReadOnly | QIODevice::Text)) {
			return false; // No se pudo abrir el archivo para lectura
		}

		QTextStream in(&file);

		// Leer los datos del archivo
		while (!in.atEnd()) {
			QString line = in.readLine();
			QStringList parts = line.split(": ");
			if (parts.size() != 2) continue; // Omitir líneas mal formadas

			QString key = parts[0].trimmed();
			QString value = parts[1].trimmed();

			if (key == "Volume") setVolume(value.toDouble());
			else if (key == "Pan") setPan(value.toDouble());
			else if (key == "Pitch") setPitch(value.toDouble());
			else if (key == "Bus1") setBus1(value.toInt());
			else if (key == "Bus2") setBus2(value.toInt());
			else if (key == "Status") setStatus(static_cast<Status>(value.toInt()));
			else if (key == "FilePath") setFilePath(value);
			else if (key == "FadeIn") setFadeIn(value.toDouble());
			else if (key == "FadeOut") setFadeOut(value.toDouble());
			else if (key == "WaitIn") setWaitIn(value.toDouble());
			else if (key == "WaitOut") setWaitOut(value.toDouble());
			else if (key == "StopAtEnd") setStopAtEnd(value.toInt() != 0);
			else if (key == "Name") setName(value);
			else if (key == "Description") setDescription(value);
			else if (key == "UserNumber") setUserNumber(value.toInt());
			else if (key == "EntryPoint") setEntryPoint(value.toDouble());
			else if (key == "ExitPoint") setExitPoint(value.toDouble());
			// Añadir más campos según sea necesario
		}

		file.close();
		return true;
	}
};

#include <QFile>
#include <QXmlStreamWriter>

void saveCueTrackToXml(const CueTrack& cueTrack, const QString& filePath) {
	QFile file(filePath);
	if (!file.open(QIODevice::WriteOnly)) {
		qWarning("No se pudo abrir el archivo para escritura.");
		return;
	}

	QXmlStreamWriter xmlWriter(&file);
	xmlWriter.setAutoFormatting(true); // Para una salida más legible

	xmlWriter.writeStartDocument();
	xmlWriter.writeStartElement("CueTrack");

	// Escribe los datos del objeto CueTrack como elementos o atributos
	xmlWriter.writeTextElement("Name", cueTrack.name());
	xmlWriter.writeTextElement("Volume", QString::number(cueTrack.volume()));
	xmlWriter.writeTextElement("Pan", QString::number(cueTrack.pan()));
	xmlWriter.writeTextElement("Pitch", QString::number(cueTrack.pitch()));
	// Añade más elementos según sea necesario

	xmlWriter.writeEndElement(); // Cierra el elemento CueTrack
	xmlWriter.writeEndDocument();

	file.close(); // No olvides cerrar el archivo
}

#include <QFile>
#include <QXmlStreamReader>

bool loadCueTrackFromXml(CueTrack& cueTrack, const QString& filePath) {
	QFile file(filePath);
	if (!file.open(QIODevice::ReadOnly)) {
		qWarning("No se pudo abrir el archivo para lectura.");
		return false;
	}

	QXmlStreamReader xmlReader(&file);

	while (!xmlReader.atEnd() && !xmlReader.hasError()) {
		// Lee el siguiente elemento
		QXmlStreamReader::TokenType token = xmlReader.readNext();
		// Si es un elemento de inicio y el nombre del elemento es el esperado
		if (token == QXmlStreamReader::StartElement) {
			if (xmlReader.name() == "filePath") {
				cueTrack.filePath = xmlReader.readElementText().toStdString();
			} else if (xmlReader.name() == "volume") {
				cueTrack.volume = xmlReader.readElementText().toInt();
			} else if (xmlReader.name() == "pan") {
				cueTrack.pan = xmlReader.readElementText().toInt();
			} else if (xmlReader.name() == "pitch") {
				cueTrack.pitch = xmlReader.readElementText().toInt();
			} else if (xmlReader.name() == "status") {
				QString statusStr = xmlReader.readElementText();
				cueTrack.status = stringToStatus(&statusStr); // Asegúrate de que esta función maneje QString correctamente
			}
			// Añade más campos según sea necesario
		}
	}

	if (xmlReader.hasError()) {
		qWarning("Error al leer el archivo XML: %s", xmlReader.errorString().toUtf8().constData());
		return false;
	}

	file.close();
	return true;
}
}



*/
#endif // CUETRACKWIDGET_H
